package com.example.navigationbaato.factory

import android.app.Activity
import android.app.Application
import android.content.Context
import android.content.Intent
import android.os.Bundle
import android.view.View
import android.widget.Toast
import com.baato.baatolibrary.models.DirectionsAPIResponse
import com.baato.baatolibrary.services.BaatoRouting
import com.example.navigationbaato.activity.NavigationActivity
import com.example.navigationbaato.models.MapBoxEvents
import com.example.navigationbaato.utilities.PluginUtilities
import com.mapbox.api.directions.v5.DirectionsCriteria
import com.mapbox.api.directions.v5.models.DirectionsResponse
import com.mapbox.api.directions.v5.models.DirectionsRoute
import com.mapbox.geojson.Point

import com.mapbox.services.android.navigation.ui.v5.NavigationView
import io.flutter.plugin.common.BinaryMessenger
import io.flutter.plugin.common.EventChannel
import io.flutter.plugin.common.MethodCall
import io.flutter.plugin.common.MethodChannel
import io.flutter.plugin.platform.PlatformView
import java.util.*

class NavigationFactory: PlatformView,
MethodChannel.MethodCallHandler,
Application.ActivityLifecycleCallbacks, EventChannel.StreamHandler {

    private val activity: Activity
    private val context: Context

    private val methodChannel: MethodChannel
    private val eventChannel: EventChannel
    private var accessToken = ""
    private var navMapDef = ""
    private var currentRoute: DirectionsRoute? = null
    private var navigationView: NavigationView? = null
    private var directionsResponse: DirectionsResponse? = null

    constructor(cxt: Context, messenger: BinaryMessenger, accessToken: String, navMapDef: String, viewId: Int, act: Activity, args: Any?)
    {
        context = cxt
        activity = act
        this.accessToken = accessToken
        this.navMapDef = navMapDef
        val arguments = args as? Map<*, *>
        if(arguments != null)
            setOptions(arguments)

        methodChannel = MethodChannel(messenger, "flutter_mapbox_navigation/${viewId}")
        eventChannel = EventChannel(messenger, "flutter_mapbox_navigation/${viewId}/events")
        eventChannel.setStreamHandler(this)
        
        activity.application.registerActivityLifecycleCallbacks(this)
        methodChannel.setMethodCallHandler(this)
    }

    companion object {

        //Config
        var initialLatitude: Double? = null
        var initialLongitude: Double? = null

        val wayPoints: MutableList<Point> = mutableListOf()
        var navigationMode =  DirectionsCriteria.PROFILE_DRIVING_TRAFFIC
        var simulateRoute = false
        var mapStyleURL: String? = null
        var navigationLanguage = "en"
        var navigationVoiceUnits = DirectionsCriteria.IMPERIAL
        var zoom = 15.0
        var bearing = 0.0
        var tilt = 0.0
        var distanceRemaining: Double? = null
        var durationRemaining: Double? = null

        var alternatives = true

        var allowsUTurnAtWayPoints = false
        var enableRefresh = false
        var voiceInstructionsEnabled = true
        var bannerInstructionsEnabled = true
        var longPressDestinationEnabled = true
        var animateBuildRoute = true
        var isOptimized = false

        var originPoint: Point? = null
        var destinationPoint: Point? = null
    }


    override fun onMethodCall(methodCall: MethodCall, result: MethodChannel.Result) {
        when (methodCall.method) {

            "buildRoute" -> {
                buildRoute(methodCall, result)
            }
            "clearRoute" -> {
                clearRoute(methodCall, result)
            }
            "startNavigation" -> {
                startNavigation(methodCall, result)
            }
            "finishNavigation" -> {
                finishNavigation(methodCall, result)
            }
            "getDistanceRemaining" -> {
                result.success(distanceRemaining)
            }
            "getDurationRemaining" -> {
                result.success(durationRemaining)
            }
            else -> result.notImplemented()
        }
    }
    private fun startNavigation(methodCall: MethodCall, result: MethodChannel.Result) {

        val arguments = methodCall.arguments as? Map<*, *>
        if(arguments != null)
            setOptions(arguments)

        startNavigation()

        if (currentRoute != null) {
            result.success(true)
        } else {
            result.success(false)
        }
    }

    private fun finishNavigation(methodCall: MethodCall, result: MethodChannel.Result) {

        if (currentRoute != null) {
            result.success(true)
        } else {
            result.success(false)
        }
        PluginUtilities.sendEvent(MapBoxEvents.NAVIGATION_FINISHED)
    }

    private fun startNavigation() {
        val intent = Intent(activity, NavigationActivity::class.java)

        //        Intent intent = new Intent(MainActivity.this, ComponentNavigationActivity.class);
        intent.putExtra("navMode", navigationMode)
        intent.putExtra("mapStyle", mapStyleURL!!)
        intent.putExtra("locale", navigationLanguage)
        intent.putExtra("Route", directionsResponse)
        intent.putExtra("origin", originPoint)
        intent.putExtra("destination", destinationPoint)
//        val lastLocation =  locationEngine?.lastLocation
        intent.putExtra("lastLocation", originPoint)
        activity.startActivity(intent)
        PluginUtilities.sendEvent(MapBoxEvents.NAVIGATION_RUNNING)
    }

    private fun buildRoute(methodCall: MethodCall, result: MethodChannel.Result) {

        val arguments = methodCall.arguments as? Map<*, *>
        if(arguments != null)
            setOptions(arguments)

            wayPoints.clear()
            var points = arguments?.get("wayPoints") as HashMap<*, *>
            for (item in points)
            {
                val point = item.value as HashMap<*, *>
                val latitude = point["Latitude"] as Double
                val longitude = point["Longitude"] as Double
                wayPoints.add(Point.fromLngLat(longitude, latitude))
            }
            getRoute(context)
            result.success(true)
    }
    private fun getRoute(context: Context) {

        if (!PluginUtilities.isNetworkAvailable(context)) {
            PluginUtilities.sendEvent(MapBoxEvents.ROUTE_BUILD_FAILED, "No Internet Connection")
            return
        }
        PluginUtilities.sendEvent(MapBoxEvents.ROUTE_BUILDING)

        originPoint = Point.fromLngLat(wayPoints[0].longitude(), wayPoints[0].latitude())
        destinationPoint = Point.fromLngLat(wayPoints[1].longitude(), wayPoints[1].latitude())
        getRoute(originPoint!!, destinationPoint!!, navigationMode)
    }
    private fun getRoute(origin: Point, destination: Point, navigationMode: String) {
        val points = arrayOfNulls<String>(2)
        points[0] = origin.latitude().toString() + "," + origin.longitude()
        points[1] = destination.latitude().toString() + "," + destination.longitude()
        BaatoRouting(context)
            .setPoints(points)
            .setAccessToken(accessToken)
            .setMode("car") //eg bike, car, foot
            .setAlternatives(false) //optional parameter
            .setInstructions(true) //optional parameter
            .withListener(object : BaatoRouting.BaatoRoutingRequestListener {
                override fun onSuccess(directionResponse: DirectionsAPIResponse) {
                    // If the routing returns a result, we take the first in the list and show the route.
                    val navResponse = directionResponse.data[0]
//                        initRouteCoordinates(navResponse.encoded_polyline)
                    val distanceInKm = navResponse.distanceInMeters / 1000
                    val time = navResponse.timeInMs / 1000
//                        btnGo.setVisibility(View.VISIBLE)
//                        bottomText.setText("Distance: " + com.baato.baatoandroiddemo.activities.ComponentNavigationActivity.df.format(distanceInKm) + " km" +
//                                "  Time: " + TimeCalculation.giveMeTimeFromSecondsFormat(time))
//                        bottomInfoLayout.setVisibility(View.VISIBLE)
                    val parseLocale = Locale(navigationLanguage)
                    val parsedNavigationResponse = BaatoRouting.getParsedNavResponse(directionResponse, navigationMode, parseLocale, context)
                    directionsResponse = DirectionsResponse.fromJson(parsedNavigationResponse)
                    currentRoute = directionsResponse?.routes()?.get(0)
                }

                override fun onFailed(t: Throwable) {
                    if (t.message != null && t.message!!.contains("Failed to connect")) Toast.makeText(context, "Please connect to internet to get the routes!", Toast.LENGTH_SHORT).show()
                }
            })
            .doRequest()
    }
    private fun setOptions(arguments: Map<*, *>)
    {
        val navMode = arguments["mode"] as? String
        if(navMode != null)
        {
            navigationMode = navMode
        }

        val simulated = arguments["simulateRoute"] as? Boolean
        if (simulated != null) {
            simulateRoute = simulated
        }

        val language = arguments["language"] as? String
        if(language != null)
            navigationLanguage = language

        val units = arguments["units"] as? String

        if(units != null)
        {
            if(units == "imperial")
                navigationVoiceUnits = DirectionsCriteria.IMPERIAL
            else if(units == "metric")
                navigationVoiceUnits = DirectionsCriteria.METRIC
        }

        mapStyleURL = arguments["mapStyleURL"] as? String
        if (mapStyleURL == null)
            mapStyleURL = navMapDef

        initialLatitude = arguments["initialLatitude"] as? Double
        initialLongitude = arguments["initialLongitude"] as? Double

        val zm = arguments["zoom"] as? Double
        if(zm != null)
            zoom = zm

        val br = arguments["bearing"] as? Double
        if(br != null)
            bearing = br

        val tt = arguments["tilt"] as? Double
        if(tt != null)
            tilt = tt

        val optim = arguments["isOptimized"] as? Boolean
        if(optim != null)
            isOptimized = optim

        val anim = arguments["animateBuildRoute"] as? Boolean
        if(anim != null)
            animateBuildRoute = anim

        val altRoute = arguments["alternatives"] as? Boolean
        if(altRoute != null)
            alternatives = altRoute

        val voiceEnabled = arguments["voiceInstructionsEnabled"] as? Boolean
        if(voiceEnabled != null)
            voiceInstructionsEnabled = voiceEnabled

        val bannerEnabled = arguments["bannerInstructionsEnabled"] as? Boolean
        if(bannerEnabled != null)
            bannerInstructionsEnabled = bannerEnabled

        var longPress = arguments["longPressDestinationEnabled"] as? Boolean
        if(longPress != null)
            longPressDestinationEnabled = longPress
    }


    private fun clearRoute(methodCall: MethodCall, result: MethodChannel.Result) {
        PluginUtilities.sendEvent(MapBoxEvents.NAVIGATION_CANCELLED)
    }
    override fun getView(): View {
        TODO("Not yet implemented")
    }

    override fun dispose() {
        TODO("Not yet implemented")
    }

    override fun onActivityCreated(activity: Activity, savedInstanceState: Bundle?) {
        TODO("Not yet implemented")
    }

    override fun onActivityStarted(activity: Activity) {
        TODO("Not yet implemented")
    }

    override fun onActivityResumed(activity: Activity) {
        TODO("Not yet implemented")
    }

    override fun onActivityPaused(activity: Activity) {
        TODO("Not yet implemented")
    }

    override fun onActivityStopped(activity: Activity) {
        TODO("Not yet implemented")
    }

    override fun onActivitySaveInstanceState(activity: Activity, outState: Bundle) {
        TODO("Not yet implemented")
    }

    override fun onActivityDestroyed(activity: Activity) {
        TODO("Not yet implemented")
    }

    override fun onListen(arguments: Any?, events: EventChannel.EventSink?) {
        TODO("Not yet implemented")
    }

    override fun onCancel(arguments: Any?) {
        TODO("Not yet implemented")
    }

}